try:  # pragma: no cover
    from celery import shared_task

    @shared_task()
    def run_check():
        from updater import check
        return check.run_check()
except ImportError:  # pragma: no cover
    pass



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import requests

try:
    from django.contrib.sites.shortcuts import get_current_site
except ImportError:  # pragma: no cover
    # django < 1.7 compatibility
    from django.contrib.sites.models import get_current_site
from django.core.mail.message import EmailMultiAlternatives
from django.template.loader import render_to_string
from django import conf, get_version

from .conf import settings


def run_check(notify=True):
    """
    Main entrypoint for all update checks. Fetches issues and updates and decides if a
    notification is sent.
    :param notify: (Optional) send notifications
    :return: tuple containing the status and the version currently in use
    """
    r = requests.get("/".join([settings.UPDATER_BASE_URL, "django.json"]))
    releases = r.json()["releases"]
    used_version = get_version()
    status = releases.get(used_version, "unknown")
    if notify and settings.UPDATER_NOTIFY and status in ["insecure", "outdated"]:
        send_notification(status, used_version)
    return status, used_version


def send_notification(status, version):
    """
    Sends a notification.
    :param status: the status of the used version "insecure" or "outdated"
    :param version: used django version
    :return: True if all notifcations have been sent successfully
    """
    # only mails are supported right now. This might change, so we go for the more generic
    # `send_notification` as method name, but use it as a proxy to send_mail
    return send_mail(status, version,
                     mail_from=conf.settings.SERVER_EMAIL, mail_to=settings.UPDATER_EMAILS)


def send_mail(status, version, mail_from, mail_to, fail_silently=False):
    """
    Sends a notification email.
    :param status: the status of the used version "insecure" or "outdated"
    :param version: used django version
    :param mail_from: mail address that is used as the sender
    :param mail_to: mail address to send the mail to
    :param fail_silently:
    :return: :bool: True if mail has been send successfully
    """
    site = get_current_site(None)
    subject = "Important: Security update for %s" % site if status == "insecure" \
        else "Update available for %s" % site
    context = {"status": status, "version": version, "site": site}
    txt_message = render_to_string("mail.txt", context=context)
    html_message = render_to_string("mail.html", context=context)
    mail = EmailMultiAlternatives(
        subject, txt_message, mail_from, mail_to)
    mail.attach_alternative(html_message, 'text/html')
    return mail.send(fail_silently=fail_silently)



# -*- coding: utf-8 -*-
from django.db import models


# this module contains an empty model to use Django's ModelAdmin to add a view to the admin
# without too much hassle
class Status(models.Model):
    pass



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import json
from django.contrib import admin
from django.conf.urls import url
from django.http import HttpResponse
from .models import Status
from updater.check import run_check
import time


class StatusAdmin(admin.ModelAdmin):

    def has_module_permission(self, request):
        # don't display the app on the index site
        return False

    def get_urls(self):
        urls = super(StatusAdmin, self).get_urls()
        return [
            url(r'^update-status/$', self.admin_site.admin_view(self.registration_status_view),
                name="update-status"),
        ] + urls

    def registration_status_view(self, request):
        request.current_app = self.admin_site.name
        status, version = run_check(notify=False)
        data = {
            "status": status,
            "version": version,
            "run_again_at": time.time() + 60 * 60 * 24
        }
        request.session["updater_admin_run"] = data
        return HttpResponse(json.dumps(data), content_type="application/json")

admin.site.register(Status, StatusAdmin)


# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.core.management.base import BaseCommand
from updater.check import run_check
import sys
import traceback


class Command(BaseCommand):

    def handle(self, *args, **options):
        try:
            status, version = run_check(notify=False)
            sys.stdout.write("Django {version} is {status}".format(status=status, version=version))
            sys.exit(0)
        except Exception as e:
            sys.stderr.write(traceback.format_exc())
            sys.exit(1)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.conf import BaseSettings, settings as django_settings


class Settings(BaseSettings):

    UPDATER_NOTIFY = getattr(django_settings, "UPDATER_NOTIFY_ON_UPDATES", True)
    UPDATER_EMAILS = getattr(django_settings, "UPDATER_EMAILS", [mail for name, mail in django_settings.ADMINS])
    UPDATER_BASE_URL = getattr(django_settings, "UPDATER_BASE_URL", "https://djangoupdater.com")

settings = Settings()